/*
 * Copyright (C) 2004, 2008 Pedro Luis Lucas Rosado
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

/* Variables globales */
int global_sock; /*Sockect usado para la destruccin de la conexin.*/

/******************************************************/
/******************************************************/

/**Determina si la cadena str termina con el sufijo suf. Devuelve 0 si las cadenas son diferentes.
 */
int str_ends_with(char *str, char *suf)
{
	int str_len=strlen(str);
	int suf_len=strlen(suf);

	char *p_str, *p_suf;

	if(str_len<suf_len) return 0;

	p_str=str+str_len;
	p_suf=suf+suf_len;

	while(p_suf!=suf && p_str!=str)
	{
		if(*p_suf!=*p_str) return 0;
		p_suf--; p_str--;
	}

	return 1;
}

/**Determina si se trata de un archivo dentro de un archivo comprimido o no. Devuelve el path al archivo comprimido o NULL si no hay un archivo comprimido en la ruta.*/
char* is_compressed(char *url)
{
	char *path, *p=strdup(url), *compressed=NULL;
	struct stat buf;

	path=p;

	while(*p!='\0' && compressed==NULL)
	{
		if(*p=='/')
		{
			*p='\0';
			printf("[is_compressed] Path %s\n", path);
			if(!stat(path, &buf))
			{
				printf("[is_compressed] El archivo existe\n");
				if(0==S_ISDIR(buf.st_mode))
				{
					printf("[is_compressed] Comprobando tipo de archivo\n");
					if(str_ends_with(path,".zip"))
					{
						compressed=strdup(path);
					}
					else if(str_ends_with(path,".tar"))
					{
						compressed=strdup(path);
					}
					else if(str_ends_with(path,".tgz"))
					{
						compressed=strdup(path);
					}
					else if(str_ends_with(path,"tar.gz"))
					{
						compressed=strdup(path);
					}
					else if(str_ends_with(path,"tar.bz2"))
					{
						compressed=strdup(path);
					}
					else if(str_ends_with(path,"tb2"))
					{
						compressed=strdup(path);
					}
				}
			}
			*p='/';
		}
		p++;
	}

	free(path);

	return compressed;
}

/******************************************************/
/******************************************************/

/**Dermina si el path est dentro del directorio donde se estn sirviendo los documentos. Es para evitar salirse al usar ..  ../../ */
static int
is_in_www_root(const char *url)
{
	char *p,*buf, *mem;
	int nivel=0;

	mem=buf=p=(char*)malloc(sizeof(char)*(strlen(url)+1));

	strcpy(p,url);

	p=strtok_r(p, "/", &buf);

	while(p!=NULL && nivel>=0)
	{
		if(strcmp(p,"..")==0) nivel--;
		else if(strcmp(p,".")!=0 && *p!='\0') nivel++;

		p=strtok_r(NULL, "/", &buf);
	}

	free(mem);

	if(nivel<0) return 0;
	return 1;
}

static void
init_sockaddr (struct sockaddr_in *name,
	       const char *hostname, unsigned short int port)
{
	  struct hostent *hostinfo;

	  name->sin_family = AF_INET;
	  name->sin_port = htons (port);
	  hostinfo = gethostbyname (hostname);
	  if (hostinfo == NULL)
	    {
	      fprintf (stderr, "Unknown host %s.\n", hostname);
	      exit (EXIT_FAILURE);
	    }
	  name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
}


static int
write_bytes_to_socket (int filedes, const char *MESSAGE, int numbytes)
{
	  int nbytes;

	//printf("%s", MESSAGE);

	  nbytes = write (filedes, MESSAGE, numbytes );
	  if (nbytes < 0)
	    {
	      perror ("write");
	      /*exit (EXIT_FAILURE);*/
	      return 0;
	    }

	  return 1;
}

static int
write_to_socket (int filedes, const char *MESSAGE)
{
	  return write_bytes_to_socket (filedes, MESSAGE, strlen (MESSAGE) );
}



static char *
read_from_socket (int filedes)
{
	  int nbytes, buftam=1;
	  char *buf=(char*)malloc(sizeof(char));
	  char buffer[1024];

	  if(buf==NULL)
	  {
	  	perror ("Memoria agotada");
	        exit (EXIT_FAILURE);
	  }

	  *buf='\0';

	  nbytes=(sizeof(buffer)-1);
	  while( nbytes==(sizeof(buffer)-1) )
	  {
	  	nbytes = read (filedes, buffer,  sizeof(buffer)-1);
	  	if (nbytes < 0)
	    	{
	      		perror ("write");
	      		/*exit (EXIT_FAILURE);*/
	      		return buf;
	    	}
	    	buftam+=nbytes;
	    	buf=(char*) realloc(buf,buftam);
	    	if(buf==NULL)
	    	{
		    	perror ("Memoria agotada");
		        exit (EXIT_FAILURE);
	    	}
		//buffer[nbytes]='\0';
	    	strncat(buf, buffer, nbytes);
	  }

	  return buf;
}

struct mime_tipo {const char *extension, *tipo_mime;};
static void
tipo_mime( char *url , char *tipo)
{
	struct mime_tipo tipos[]=
		{
			{".html","text/html"},
			{".htm", "text/html"},
			{".gif", "image/gif"},
			{".png", "image/png"},
			{".jpg", "image/jpeg"},
			{".tif", "image/tiff"},
			{".xml", "text/xml"},
			{".xhtml", "application/xhtml+xml"},
			{".txt", "text/plain"},
			{".pdf", "application/pdf"},
			{NULL, NULL}
		};
	struct stat st;
	char *tipo_p=strrchr(url, '.');
	int i=0;

	for(i=0;tipos[i].extension!=NULL; i++)
	{
		if( !strcmp(tipo_p, tipos[i].extension ) )
		{
			strcpy(tipo, tipos[i].tipo_mime);
			return;
		}
	}

	if(stat(url, &st)==0)
		if( S_ISDIR(st.st_mode)==0 )
		{
			strcpy(tipo, "application/octet-stream");
			return;
		}

	strcpy(tipo, "text/html");
}

/* Devuelve 0 si el fichero es ejecutable por el usuario y no es un directorio*/
static int
is_exec_and_not_dir(const char *url)
{
	struct stat st;

	if(stat(url, &st)==0)
	{
		if( S_ISDIR(st.st_mode)==0 )
		{
			return access(url, X_OK);
		}
	}

	return 1;
}

/**Crea una nueva cadena resultado de encadenar cad1 y cad2.*/
static char *
str_new(const char *cad1, const char *cad2)
{
	char *aux;
	if(cad2!=NULL) aux=malloc(sizeof(char)*(strlen(cad1)+strlen(cad2)+1));
	else aux=malloc(sizeof(char)*(strlen(cad1)+1));
	if( aux== NULL ) return NULL;
	strcpy(aux,cad1);
	if(cad2!=NULL) strcat(aux,cad2);
	return aux;
}

static void
exec_prog( const char *file, char *const argv[], char *const env[], FILE **std_in, FILE **std_out)
{
	int out[2], in[2];

	pipe(in);
	pipe(out);

	*std_in=fdopen(in[0],"r");
	*std_out=fdopen(out[1],"w");

	if( fork()==0 )
	{
		char *f;

		close(in[0]);
		close(out[1]);

		dup2(in[1],STDOUT_FILENO);
		dup2(out[0],STDIN_FILENO);

		close(in[1]);
		close(out[0]);

		f=strrchr(file,'/');
		*f='\0';
		f++;

		chdir(file);

		if( execve(f,argv,env)<0 )
		{
			char *error="Content-type= text/html\n\r\n\r<html>Error.</html>";
			write (STDOUT_FILENO, error, strlen(error));
		}

		close(STDOUT_FILENO);
		close(STDIN_FILENO);

		exit(EXIT_SUCCESS);
	}

	close(in[1]);
	close(out[0]);
}

static char *
find_post_data_in_head(const char *head_data)
{
	char *aux;
	if( (aux=strstr(head_data,"\r\n\r\n")) != NULL)
		return aux+4;
	else if((aux=strstr(head_data,"\n\n")) != NULL)
		return aux+2;
	return NULL;
}

/*Procesa un cgi.
 get_or_post: si vale 'g' se procesa un mtodo get. Si vale 'p' se procesa un mtodo post.
 */
static void
cgi(char get_or_post, char *url, const char *cgi_values, const char *head_data, int sock)
{
	FILE *std_in, *std_out;
	char *argv[2], **env, buf[4096];
	int nbytes;

	argv[0]=url;
	argv[1]=NULL;

	/*Defino el entorno con el que se va a ejecutar el cgi*/
	env=(char **)malloc(sizeof(char*)*30);
	nbytes=0;
	/*Paso el path*/
	env[nbytes++]=str_new("PATH=",getenv("PATH"));
	/*HTTP_HOST=localhost*/
	//env[nbytes++]=str_new("HTTP_HOST=localhost",NULL);
	/*HTTP_USER_AGENT=Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.4.1) Gecko/20031030 Epiphany/1.0.4*/
	//env[nbytes++]=str_new("HTTP_USER_AGENT=Morzilla",NULL);
	/*HTTP_ACCEPT=text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,video/x-mng,image/png,image/jpeg,image/gif;q=0.2,*/
	//env[nbytes++]=str_new("HTTP_ACCEPT=text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,video/x-mng,image/png,image/jpeg,image/gif;q=0.2",NULL);
	/*HTTP_ACCEPT_LANGUAGE=en,en;q=0.5*/
	//env[nbytes++]=str_new("HTTP_ACCEPT_LANGUAGE=en,en;q=0.5",NULL);
	/*HTTP_ACCEPT_ENCODING=gzip,deflate*/
	//env[nbytes++]=str_new("HTTP_ACCEPT_ENCODING=gzip,deflate",NULL);
	/*HTTP_ACCEPT_CHARSET=ISO-8859-1,utf-8;q=0.7,*;q=0.7*/
	//env[nbytes++]=str_new("HTTP_ACCEPT_CHARSET=ISO-8859-1,utf-8;q=0.7,*;q=0.7",NULL);
	/*HTTP_KEEP_ALIVE=300*/
	//env[nbytes++]=str_new("HTTP_KEEP_ALIVE=300",NULL);
	/*HTTP_CONNECTION=keep-alive*/
	//env[nbytes++]=str_new("HTTP_CONNECTION=keep-alive",NULL);
	/*HTTP_REFERER=http://localhost/texto.html*/
	/*HTTP_CACHE_CONTROL=max-age=0*/
	/*SERVER_SOFTWARE=*/
	env[nbytes++]=str_new("SERVER_SOFTWARE=","A");
	/*SERVER_SIGNATURE=*/
	env[nbytes++]=str_new("SERVER_SIGNATURE=","A");
	/*SERVER_NAME=localhost*/
	//env[nbytes++]=str_new("SERVER_NAME=localhost",NULL);
	/*SERVER_ADDR=127.0.0.1*/
	//env[nbytes++]=str_new("SERVER_ADDR=127.0.0.1",NULL);
	/*SERVER_PORT=80*/
	//env[nbytes++]=str_new("SERVER_PORT=80",NULL);
	/*REMOTE_ADDR=127.0.0.1*/
	//env[nbytes++]=str_new("REMOTE_ADDR=127.0.0.10",NULL);
	/*REMOTE_PORT=33111*/
	//env[nbytes++]=str_new("REMOTE_PORT=33111",NULL);
	/*SERVER_PROTOCOL=HTTP/1.1*/
	env[nbytes++]=str_new("SERVER_PROTOCOL=HTTP/1.1",NULL);
	/*REQUEST_URI=*/
	//env[nbytes++]=str_new("REQUEST_URI=",url);
	/*SCRIPT_NAME=*/
	env[nbytes++]=str_new("SCRIPT_NAME=",url);
	/*Paso el protocolo GATEWAY_INTERFACE*/
	env[nbytes++]=str_new("GATEWAY_INTERFACE=","CGI/1.1");
	/*Paso el protocolo REQUEST_METHOD*/
	if(get_or_post=='p')
		env[nbytes++]=str_new("REQUEST_METHOD=","POST");
	else
		env[nbytes++]=str_new("REQUEST_METHOD=","GET");
	/*Paso el QUERY_STRING*/
	env[nbytes++]=str_new("QUERY_STRING=",cgi_values);
	/*Paso el CONTENT_LENGTH*/
	if(get_or_post=='p')
	{
		char *aux;
		if((aux=find_post_data_in_head(head_data)) != NULL)
		{
			sprintf(buf,"%d", strlen(aux));
		}
		else
			strcpy(buf,"0");

		env[nbytes++]=str_new("CONTENT_LENGTH=",buf);
	}
	/*Fin de los parmetros*/
	env[nbytes++]=NULL;

	printf("Procesando cgi\n\n");
	int i=0;
	for(;env[i]!=NULL; i++)
		printf("Entorno: %s\n", env[i]);

	exec_prog(url,argv,env, &std_in, &std_out);

	/*Si es un post, se le enva la informacin*/
	if(get_or_post=='p')
	{
		char *aux;
		if((aux=find_post_data_in_head(head_data)) != NULL)
		{
			fwrite(aux, strlen(aux),1, std_out);
			fclose(std_out);
		}
	}

	nbytes = sizeof(buf);
	while( nbytes==(sizeof(buf)) && !feof(std_in) )
	{
		nbytes = fread(buf, 1, sizeof(buf),std_in);
		if( nbytes >0 )
		{
			write_bytes_to_socket (sock, buf, nbytes);
		}
	}


	fclose(std_in);

	for(nbytes=0; env[nbytes]!=NULL; nbytes++)
	{
		free(env[nbytes]);
	}

	free(env);

	printf("CGI procesado\n");
}

static void enviar_url_respuesta_archivo(int sock, const char *url);
static void enviar_url_respuesta_zip(int sock, const char *url, const char *compressed);
static void enviar_url_respuesta_tgz(int sock, const char *url, const char *compressed);
static void enviar_url_respuesta_ejecutar_descompresor(int sock, const char *url, const char *compressed, const char *descompresor);

/**Enva el archivo indicado por url al cliente. Si compressed es no nulo se considera que el archivo est dentro de un archivo comprimido.
 */
static void
enviar_url_respuesta(int sock, const char *url, char *compressed)
{
	if(compressed==NULL)
	{	/*El archivo no est comprimido:*/
		enviar_url_respuesta_archivo(sock, url);
		return;
	}

	printf("[enviar_url_respuesta] ARchivo comprimido %s\n", compressed);
	/*El archivo est comprimido:*/
	if(str_ends_with(compressed, ".zip"))
	{
		printf("[enviar_url_respuesta] ARchivo comprimido zip\n");
		enviar_url_respuesta_zip(sock, url, compressed);
	}
	else if(
		str_ends_with(compressed, ".tar.gz")
		||
		str_ends_with(compressed, ".tgz")
		)
	{
		printf("[enviar_url_respuesta] ARchivo comprimido tgz\n");
		enviar_url_respuesta_tgz(sock, url, compressed);
	}
	else if(
		str_ends_with(compressed, ".tar.bz2")
		||
		str_ends_with(compressed, ".tb2")
		)
	{
		printf("[enviar_url_respuesta] ARchivo comprimido tgz\n");
		enviar_url_respuesta_ejecutar_descompresor(sock, url, compressed, "tar -xjOf");
	}
	else if( str_ends_with(compressed, ".tar"))
	{
		printf("[enviar_url_respuesta] ARchivo comprimido tgz\n");
		enviar_url_respuesta_ejecutar_descompresor(sock, url, compressed, "tar -xOf");
	}

	free(compressed);
}

static void
enviar_url_respuesta_archivo(int sock, const char *url)
{
	FILE *in=NULL;
	char buf[4096];
	int nbytes;
	struct stat st;

	if(stat(url, &st)==0)
	{
		if( S_ISDIR(st.st_mode)==0 )
		{
			in=fopen(url,"rb");
		}
	}

	if( in==NULL )
	{
		char *index=(char*)malloc(sizeof(char)*(strlen(url)+13));
		sprintf(index, "%s/index.html", url);
		in=fopen(index, "rb");
		if(in==NULL)
		{
			printf("Archivo index.html no encontrado\n");
			return;
		}
	}

	printf("Enviando archivo %s\n", url);

	nbytes=sizeof(buf);
	while( nbytes==(sizeof(buf)) )
	{
		nbytes=fread(buf, sizeof(char),sizeof(buf),in);
		printf("Bytes leidos %d\n", nbytes);
		if(!write_bytes_to_socket(sock, buf, nbytes)) break;
	}

	fclose(in);
}


static void
enviar_url_respuesta_ejecutar_descompresor(int sock, const char *url, const char *compressed, const char *descompresor)
{
	FILE *in=NULL;
	char buf[4096], *command;
	int nbytes;

	command=malloc(sizeof(char)*(strlen(descompresor)+strlen(url)+strlen(compressed)+4));

	sprintf(command,"%s %s %s", descompresor, compressed, (url+strlen(compressed)+1));

	printf("[enviar_url_respuesta_ejecutar_descompresor] Comando %s\n", command);
	in=popen(command,"r");

	if( in==NULL )
	{
		char *index=(char*)malloc(sizeof(char)*(strlen(url)+13));
		sprintf(index, "%s/index.html", url);
		in=fopen(index, "rb");
		if(in==NULL)
		{
			printf("Archivo index.html no encontrado\n");
			return;
		}
	}

	printf("Enviando archivo %s\n", url);

	nbytes=sizeof(buf);
	while( nbytes==(sizeof(buf)) )
	{
		nbytes=fread(buf, sizeof(char),sizeof(buf),in);
		printf("Bytes leidos %d\n", nbytes);
		if(!write_bytes_to_socket(sock, buf, nbytes)) break;
	}

	pclose(in);
	free(command);
}


static void
enviar_url_respuesta_zip(int sock, const char *url, const char *compressed)
{
	enviar_url_respuesta_ejecutar_descompresor(sock, url, compressed, "unzip -p");
}

static void
enviar_url_respuesta_tgz(int sock, const char *url, const char *compressed)
{
	enviar_url_respuesta_ejecutar_descompresor(sock, url, compressed, "tar -xzOf");
}

/*
 Devuelve el url, los valores pasados por el protocolo cgi y escribe la cabecera.
 */
static void
cabecera(const char *buf, char **_url, char **cgi_values, char **head_data, int sock)
{
	char *url, *protocolo;
	char tipo[1024];
	//struct stat st;

	url=strchr(buf,' ');
	*url='.';
	*_url=url;

	*head_data=strchr(buf,'\n');
	if(head_data!=NULL) *head_data=*head_data+1;

	protocolo=strpbrk(url," \n");
	*protocolo='\0';
	protocolo++;

	*cgi_values=strchr(url,'?');
	if( *cgi_values!=NULL && *cgi_values<protocolo)
	{
		**cgi_values='\0';
		*cgi_values=*cgi_values+1;
	}
	else
		*cgi_values=NULL;

	protocolo=strstr(protocolo, "HTTP/");

	if( protocolo==NULL )
	{
		/*No se ha indicado el protocolo.*/
		/*Se debe devolver la pgina*/
		return;
	}

	/*Se debe devolver la cabecera de respuesta*/
	write_to_socket(sock,"HTTP/1.0 200 OK\r\n");
	/*write_to_socket(sock,"Content-type: text/html\r\n");*/
	if( is_exec_and_not_dir(url) )
	{
		write_to_socket(sock,"Content-type: ");
		tipo_mime(url, tipo);
		write_to_socket(sock, tipo);
		write_to_socket(sock,"\r\n\r\n");
	}
	/*write_to_socket(sock,"Content-Length: ");
	if( stat(url,&st) < 0 )
	{
		write_to_socket(sock,"0");
	}
	else
	{
		sprintf(tipo,"%d", st.st_size);
		write_to_socket(sock,tipo);
	}
	write_to_socket(sock,"\r\n");*/
}


/**Procesa la respuesta para peticiones GET, POST.
 * Si tipo vale:
 * 'g' se procesa una peticin get
 * 'p' se procesa una peticin post
 */
static void
procesar_get_post(char tipo, const char *buf, int sock)
{
	char *url, *cgi_values, *head_data, *compressed;

	cabecera(buf, &url, &cgi_values, &head_data, sock);

	if( is_in_www_root(url)>0 )
	{
		compressed=is_compressed(url);
		if( compressed!=NULL || is_exec_and_not_dir(url) )
		{
			printf("[procesar_get_post] Es un archivo comprimido %s\n", compressed);
			enviar_url_respuesta(sock, url, compressed);
		}
		else
			cgi(tipo, url, cgi_values, head_data, sock);
	}
}


static void
procesar_get(const char *buf, int sock)
{
	procesar_get_post('g', buf, sock);
}

static void
procesar_post(const char *buf, int sock)
{
	procesar_get_post('p', buf, sock);
}

static void
procesar_head(const char *buf, int sock)
{
	char *url, *cgi_values, *head_data;

	cabecera(buf, &url, &cgi_values, &head_data, sock);
}

static void
procesar_respuesta(const char *buf, int sock)
{
	if(!strncmp(buf,"GET",3))
		procesar_get(buf,sock);
	else
	if(!strncmp(buf,"HEAD",4))
		procesar_head(buf,sock);
	else
	if(!strncmp(buf,"POST",4))
		procesar_post(buf,sock);
	else
	{
		write_to_socket(sock,"HTTP/1.0 501 Not Implemented\r\n\r\n");
	}
}

static void
sigchld_handler(int signum)
{
	int pid, status;

	while(1)
	{
		pid = waitpid(WAIT_ANY, &status, WNOHANG);

		if(pid<0 || pid==0) break;
	}
}

static void cerrar_socket(int exit_val, void *param);

static void
sigint_handler(int signum)
{
	printf("[sigkill_handler]\n");
	cerrar_socket(signum, (void *)global_sock);
	exit(EXIT_SUCCESS);
}

/**Esta funcin cierra el socket de conexin. Es usada por la funcin on_exit.*/
static void cerrar_socket(int exit_val, void *param)
{
	int sock=(int)param;
	printf("[cerrar_socket] Se procede a cerrar el socket. Valor de salida %d\n", exit_val);
	shutdown(sock, 2);
}

int
main (int argn, char *argv[])
{

	  int sock, size, sock_cliente;
	  struct sockaddr_in servername, clientname;
	  char *buf;

	  if(argn<3)
	  {
	  	printf(	"%s ip puerto\nEs un simple servidor http.\nip\tla ip que se desea"
	  		"servir.\npuerto\tel puerto en el que se desea que el servidor se instale.\n"
	  		, argv[0]);
	  	return 0;
	  }

	  /* Create the socket.   */
	  sock = socket (PF_INET, SOCK_STREAM, 0);
	  global_sock = sock;
	  //on_exit(cerrar_socket, (void*)sock);
	  if (sock < 0)
	    {
	      perror ("socket (client)");
	      exit (EXIT_FAILURE);
	    }
	  init_sockaddr (&servername, argv[1], atoi(argv[2]));
	  size=bind(sock,(struct sockaddr *) &servername, sizeof(servername));
	  if(size<0)
	  {
	  	perror ("bind");
	        exit (EXIT_FAILURE);
	  }


	   if (listen (sock, 40) < 0)
	    {
	      perror ("listen");
	      exit (EXIT_FAILURE);
	    }

	/* Capto la seal de terminacin de los procesos */
	signal(SIGCHLD, sigchld_handler );
	signal(SIGINT, sigint_handler );

	/* Conectar con el cliente*/
	while(1)
	{
		sock_cliente=accept (sock, (struct sockaddr *) &clientname, &size);
		if(sock_cliente>=0 && 0==fork())
		 {

			buf=read_from_socket (sock_cliente);
			printf( ":::::::::::::::::::::::cliente:::::::::::::::::::::::\n"
				"%s\n>>>>>>>>>>cliente<<<<<<<<<<<<\n",buf);
			fflush(stdout);
			procesar_respuesta(buf, sock_cliente);
		  	printf("Respuesta enviada\n");
		  	close(sock_cliente);
		  	free(buf);
		  	exit(EXIT_SUCCESS);
		 }
		 close(sock_cliente);
	}
	close (sock);
}
